@@ -4,13 +4,21 @@
 # Licensed under the AGPLv3: https://www.gnu.org/licenses/agpl-3.0.html
 # Copyright © 2019-2021 Gonzo Investigative Journalism Agency, LLC
 #            <gonzo.development@protonmail.ch>
-#           © 2019-2021 Richard Machado <rmlibre@riseup.net>
+#           © 2019-2023 Richard Machado <rmlibre@riseup.net>
 # All rights reserved.
 #
 
 
-from init_tests import *
+from test_initialization import *
+from test_commons import *
+from test_gentools import *
 from test_generics import *
-from test_ciphers import *
 from test_randoms import *
+from test_ciphers import *
+from test_keygens import *
+from test_databases import *
+from test_time_to_live import *
+
+
+__all__ = sorted({n for n in globals() if n.lower().startswith("test")})
 
